/*
 * Copyright 2007-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com) 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.unitestng;

import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

public class FileUtil
{
    public static void collectFilesInSubdir( List<File> files, File dir, FilenameFilter filter )
    {
        files.addAll( Arrays.asList( dir.listFiles( filter ) ) );
        for( File d : dir.listFiles( dirFilter ) )
        {
            collectFilesInSubdir( files, d, filter );
        }
    }

    public static boolean deleteRecursively( File dir )
    {
		File[] files = dir.listFiles();
		if (files == null)	return dir.delete();
		
		boolean res = true;
		for (File f: files) {
			if (f.isDirectory())	res &= deleteRecursively(f);
			else res &= f.delete();
		}
		res &= dir.delete();
		
		return res;
    }

    public static int calculateNumberOfLines( File file ) throws IOException
    {
        LineNumberReader file_reader = null;
        try {
	        file_reader = new LineNumberReader( new FileReader( file ) );
	        while( file_reader.ready() )
	        {
	            file_reader.readLine();
	        }
	        return file_reader.getLineNumber();
        } finally {
        	if (file_reader != null)	file_reader.close();
        }
    }

    public static class MaskFilenameFilter implements FilenameFilter
    {
        private String acceptMask;
        private String rejectMask;
    
        public MaskFilenameFilter( String acceptMask, String rejectMask )
        {
            this.acceptMask = acceptMask;
            this.rejectMask = rejectMask;
        }
    
        public MaskFilenameFilter( String acceptMask )
        {
            this( acceptMask, null );
        }
    
        public boolean accept( File dir, String name )
        {
            return name.matches( acceptMask )
                && (  rejectMask == null
                   || !name.matches( rejectMask )
                   );
        }
    }

    public static class FilePathComparator implements Comparator<File>
    {
        public int compare( File f1, File f2)
        {
            try
            {
                return f1.getCanonicalPath().compareTo( f2.getCanonicalPath() );
            } catch( IOException e ) {
                return 0;
            }
        }
    }

    public static final FileFilter dirFilter
    = new FileFilter()
        {
            public boolean accept( File f )
            {
                return f.isDirectory()
                    && !f.getName().matches( "CVS|\\.svn" );
            }
        };
}
